package Libs::API;

#####################################

=head1 COPYRIGHT

  Copyright Notice
  ================

  Copyright 2004 Neuronix

  Copying Permission
  ------------------

  This script is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

  ********************************************************************

=head1 INFO

  Script Info
  ===========

  Author        : Neuronix
                     aka Nikolay Viguro

  Project       : nX - Engine (Codename: XEN)
                    This is CMS engine, writted on Perl language.

  Script        : API.pm
                    This is main package.
                    Provide primary API functions.

  Started      : 19/01/2004

=cut

#########################################

require 5.004;        # Need perl dist >= 5.004

use strict;
use DBI;
use Digest::MD5 qw/md5_hex/;

use vars qw/$VERSION $skin $siteTitle $dbDriver $dbUser $dbPass $dbPrefix $dbName $dbHost $dbPort $dbh %config %modList $localtime/;

BEGIN
{
        eval 'require "Config/Main.cgi";';
        print "Location: install.cgi\n\n" if $@;
}

$VERSION = "0.6.3-test"; # Version of engine and API

our @WeekDays = ('', '', '', '', '', '', '');
our @Months = ('','','','','','','','','','','','');
our $timeformat = "%d/%B/%Y%n%H:%M";

############## End Init ###################
########################################

sub new  # The constructor
{
    my($class, %conf) = @_;
    my $self = bless {
              initMods => 0,
              readConf => 1,
                               }, $class;

    $self->_dbConnect();

    %config = $self->readConfig("Config") if $conf{'readConf'};

    $skin = $self->getSkin();

    $self->initAllModules("Modules") if $conf{'initMods'};
    $self->initAllModules("Skins") if $conf{'initMods'};

    $localtime = $self->LocalDate("%d/%B/%Y %H:%M");

    return $self;
}

########### End Constructor ###############
########################################

sub DESTROY { }

#########################################

sub initAllModules
{
    my($self, $dir) = @_;
    my @notActive;
    my @modList;
    my $line;

    #### Searching not active modules
    ##############################

    opendir (MOD, $dir) || print "Content-Type: text/html\n\n<h2>Fatal error:</h2> When try to open <b>$dir</b>: $!\n";
     my @content = readdir(MOD);
        foreach $line (@content)
           {
            next if $line =~ /^.$/ || $line =~ /^..$/;
            $self->db_get_hash_query("SELECT * FROM ${dbPrefix}_$dir WHERE BlockName=\"$line\"","BlockName, Active");

            if($_{'Active'} eq '') { $modList{$line} = 0; push(@notActive, $line); }
            else { $modList{$_{'BlockName'}} = $_{'Active'}; push(@modList, $_{'BlockName'}); }
            %_ = ();
           }
    closedir(MOD);

    #### Writing unknown modules to DB
    #################################

    foreach my $line (@notActive)
     {
      $self->db_get_hash_query("SELECT Number FROM ${dbPrefix}_Modules","Number, Active");
      my $num = $_{'Number'}+1;
      $self->db_do_query("INSERT INTO ${dbPrefix}_$dir (Number, BlockName, Date) VALUES (\"$num\", \"$line\", \"$localtime\")") if $dir ne "Modules";

        if($dir eq "Modules")
        {
          $self->db_do_query("INSERT INTO ${dbPrefix}_$dir (Number, BlockName, Date, Type) VALUES (\"$num\", \"$line\", \"$localtime\", \"Mod\")");
          $self->versionCheck($line, 1);

           my $totalC = $self->db_rows_count("SELECT ID FROM ${Libs::API::dbPrefix}_Module_Contact");
                 $totalC++;

                 #        (23.11.04 Nix)
                  $self->db_do_query("INSERT INTO ${Libs::API::dbPrefix}_Module_Contact VALUES (\"$totalC\", \"$localtime\", \"\",
                         \"\", \"     <a href=admin.cgi?do=Modules><b><i>$line</i></b></a>\", 1, 0, \"\")");
        }

      %_ = ();
     }

    ####      (23.11.04 Nix)
    #######################################################

     my $sth = $dbh->prepare("SELECT * FROM ${dbPrefix}_Modules");
             eval '$sth->execute();';
             print "Content-Type: text/html\n\n<h2>Fatal error:</h2> When try to <b>execute query</b>: ".$DBI::errstr."\n" if $@;

               while (my $ref = $sth->fetchrow_hashref())
                 {
                     opendir (MOD, "Modules/$ref->{'BlockName'}") ||
                     $self->db_do_query("DELETE FROM ${dbPrefix}_Modules WHERE BlockName=\"$ref->{'BlockName'}\"");
                     closedir(MOD);
                 }
           $sth->finish();

      ###    (23.11.04 Nix)
      ###################################################

      if($config{'CheckUpdates'})
      {
        open (IT, "<Config/InstallTime.cgi") || print "Content-Type: text/html\n\n<h2>Fatal error:</h2> When try to open <b>InstallTime</b>: $!\n";

        my $installTime = <IT>;
        chomp $installTime;
        my $sTime = time-$installTime;

        close (IT);

      if ($sTime >= 2592000) # 259200 =~ 30 .
      {
        my($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime($installTime);
        $mon++;
        my $yearl = $year + 1900;
        if ($year > 99) { $year -= 100; }
        my $monl  = sprintf('%02d', $mon);

        if(length($mday) == 1) { $mday = "0$mday"; }

           my $totalC = $self->db_rows_count("SELECT ID FROM ${Libs::API::dbPrefix}_Module_Contact");
                 $totalC++;

                 #      (23.11.04 Nix)
                  $self->db_do_query("INSERT INTO ${Libs::API::dbPrefix}_Module_Contact VALUES (\"$totalC\", \"$localtime\", \"\",
                         \"\", \"   <b>$monl\/$mday\/$yearl</b>! 
                         <a href=http://nxengine.hollosite.com><\/a> ...\", 1, 0, \"\")");

            $installTime = time;

            open (IT, ">Config/InstallTime.cgi") || print "Content-Type: text/html\n\n<h2>Fatal error:</h2> When try to open <b>InstallTime</b>: $!\n";
             print IT $installTime;
            close (IT);
       }
     }

      ###################################################

   return (@notActive, @modList, %modList);
}

#==========================================================

sub getSkin  # Getting name of skin at $skin variable.
{
  my $self = shift;
  my $tmp;

     my $sth = $dbh->prepare("SELECT * FROM ${dbPrefix}_Skins");
             eval '$sth->execute();';
             print "Content-Type: text/html\n\n<h2>Fatal error:</h2> When try to <b>execute query</b>: ".$DBI::errstr."\n" if $@;

               while (my $ref = $sth->fetchrow_hashref())
                 {
                    $tmp = $ref->{'BlockName'} if $ref->{'Active'};
                 }
           $sth->finish();

  return $tmp;
}

#==========================================================

sub checkBans # Checking bans for engine.
{
    my $self = shift;
    my $ip;

         $ip = $ENV{'REMOTE_ADDR'};

     my $sth = $dbh->prepare("SELECT * FROM ${dbPrefix}_Bans");
             eval '$sth->execute();';
             print "<h2>Fatal error:</h2> When try to <b>execute query</b>: ".$DBI::errstr."\n" if $@;

               while (my $ref = $sth->fetchrow_hashref())
                  {
                       return ($ref->{'Reason'}, $ref->{'ExcludePages'}) if $ip =~ /$ref->{'IP'}/;
                  }

           $sth->finish();

   return 0;
}

#==========================================================

sub singleInit
{
    my($self, $dir, $admin) = @_;

    #### Open modules directory and include all files with .pm extension
    ###########################################################

    $self->db_get_hash_query("SELECT * FROM ${dbPrefix}_Modules WHERE BlockName=\"$dir\";","BlockName, Active");

    if(!$admin)
     { opendir (MOD, "Modules/$dir") || print "<h2>Fatal error:</h2> When try to open <b>Modules/$dir</b>: $!\n"; }
    else
     { opendir (MOD, "Modules/$dir/admin") || print "<h2>Fatal error:</h2> When try to open <b>Modules/$dir/admin</b>: $!\n"; }

     my @content = readdir (MOD);
        foreach my $line (@content)
           {
            next if $line !~ /^.*?.pm$/;
            if($_{'Active'}) { if(!$admin)
              {
                eval 'require "Modules/$dir/$line";';  #  .
                print "<br><div align=center>
                        <h2> </h2><br>,   <b>Modules/$dir/$line</b>,   
                        <br>   .<br><br>$@</div>" if $@;
              }
               else
              {
               eval 'require "Modules/$dir/admin/$line";';   #  .
               print "<br><div align=center>
                        <h2> </h2><br>,   <b>Modules/$dir/admin/$line</b>,   
                        <br>   .<br><br>$@</div>" if $@;
              }
           }
#           else
#            {
#             print "<br><center><h2> </h2><br> ,  .
#                       <br>  .</font>\n" if $line =~ /.*?.pm/;
#             }
           }
     closedir(MOD);

     %_ = ();

   return 0;
}

#==========================================================

sub readConfig
{
  my ($self, $where) = @_;
  my %config;

     my $sth = $dbh->prepare("SELECT * FROM ${dbPrefix}_$where");
             eval '$sth->execute();';
             print "Content-Type: text/html\n\n<h2>Fatal error:</h2> When try to <b>read config</b>: ".$DBI::errstr."\n" if $@;

               while (my $ref = $sth->fetchrow_hashref())
                 {
                    $config{$ref->{'Name'}} = $ref->{'Param'};
                  }

           $sth->finish();

   return %config;
}

#==========================================================

sub printHeader
{
    my($self, $pageName) = @_;
    my $globPrint = "Content-Type: text/html\n\n";

    my @content = $self->readFile("Skins/$skin/header.html");

        foreach my $line (@content)
          {
            $line =~ s/__PAGENAME__/$pageName/eg;
            $line =~ s/__SKIN__/$skin/eg;

            $globPrint .= $line;
          }

        my @content = $self->readFile("Skins/$skin/myHeader.html");

        foreach my $line (@content)
          {
            $globPrint .= $line;
          }

      $globPrint .= "</head>\n\n";

      ###     ALW- (27.11.04 Nix)
      ###################################################

     my $sth = $dbh->prepare("SELECT * FROM ${dbPrefix}_Modules ORDER BY \"Position\" ASC");
             eval '$sth->execute();';
             print "Content-Type: text/html\n\n<h2>Fatal error:</h2> When try to <b>execute query</b>: ".$DBI::errstr."\n" if $@;

               while (my $ref = $sth->fetchrow_hashref())
                 {
                     #-------------------------------------

                     my $name = $ref->{'BlockName'};
                     my $a = $ref->{'Active'};

                     if($a && $ref->{'IsAlwLoad'})
                     {
                       eval '$self->singleInit($name);';
                       eval '$self->$name();';

                         #       . (27.11.04 Nix)

                       $globPrint .= "<br><div align=center>
                                <h2> </h2><br>,   <b>$name</b>,   
                                <br>   .<br><br>$@</div>" if $@;
                     }

                     #-------------------------------------
                 }

           $sth->finish();

   return $globPrint;
}

#=============================================================

sub printFooter
{
  my $self = shift;
  my $globPrint;

  my @content = $self->readFile("Skins/$skin/footer.html");

         foreach my $line (@content)
          {
            $line =~ s/__XEN_VERSION__/$VERSION/eg;
            $line =~ s/__SKIN__/$skin/eg;

            $globPrint .= $line;
          }

  return $globPrint;
}

#=============================================================

sub readFile
{
  my ($self, $file) = @_;
  my @keys = keys %config;
  my @contentOut;

    open(FILE, "<$file") || print "<h2>Fatal error:</h2>When try to open <b>$file</b>: $!\n";
       my @content = <FILE>;

       foreach my $line (@content)
       {
          $line =~ s/__SKIN__/$skin/eg;

          foreach my $keys (@keys) #   .
          {
            $line =~ s/__${keys}__/$config{$keys}/g;
          }

          push(@contentOut, $line);
       }

    close(FILE);

  return @contentOut;
}

#=============================================================

sub printBody
{
  my ($self, $blockLeft, $blockRight, $blockCenter) = @_;
  my $globPrint;

  my %cke = $self->getCookies();
  my $user = $cke{'login'};
        $user = "" if $user eq "";

  my @content = $self->readFile("Skins/$skin/body.html");

         foreach my $line (@content)
          {
            $line =~ s/__BLOCK_LEFT__/$blockLeft/eg;
            $line =~ s/__BLOCK_RIGHT__/$blockRight/eg;
            $line =~ s/__BLOCK_CENTER__/$blockCenter/eg;
            $line =~ s/__SKIN__/$skin/eg;
            $line =~ s/__LOCALDATE__/$self->LocalDate($timeformat)/eg;
            $line =~ s/__USER__/$user/eg;

            $globPrint .= $line;
          }

  return $globPrint;
}

#=============================================================

sub printBlock
{
  my ($self, $blockTitle, $blockContent, $type) = @_;
  my $globPrint;
  my $file = "block-$type.html";

  my @content = $self->readFile("Skins/$skin/$file");

         foreach my $line (@content)
          {
            $line =~ s/__BLOCK_NAME__/$blockTitle/eg;
            $line =~ s/__CONTENT__/$blockContent/eg;
            $line =~ s/__SKIN__/$skin/eg;

            $globPrint .= $line;
          }

  return $globPrint;
}

#=============================================================

sub printSideBlock
{
  my ($self, $side, $onIndex, $noMod) = @_;
  my $globPrint;

     my $sth = $dbh->prepare("SELECT * FROM ${dbPrefix}_${side}Block ORDER BY \"Position\" ASC");
             eval '$sth->execute();';
             print "Content-Type: text/html\n\n<h2>Fatal error:</h2> When try to <b>execute query</b>: ".$DBI::errstr."\n" if $@;

               while (my $ref = $sth->fetchrow_hashref())
                 {
                     my $name = $ref->{'BlockName'};
                     my $type = $ref->{'Type'};
                     my $tmp;

                     next if $ref->{'BlockName'} =~ $noMod && $noMod; #      (  -).

                     $self->db_get_hash_query("SELECT Active, IsAlwLoad FROM ${dbPrefix}_Modules WHERE BlockName=\"$name\"",
                     "Active, IsAlwLoad") if $type eq "Mod";

                     my $a = $_{'Active'};

                     if($type eq "Mod") { $self->singleInit($name) if $a; } else {                                                  # 
                     if($self->singleInit("Blocks")) { next; } else { $tmp .= $self->Blocks($name, $onIndex); } }     # PM-.

                    if($onIndex && $type eq "Mod") {  eval '$tmp .= $self->$name() if $a;';  }                                 # 
                    else            {  eval '$tmp .= $self->$name() if !$ref->{"OnIndex"} && $type eq "Mod" && $a;'; }  # .

                    $tmp = "<br><div align=center>
                                <h2> </h2><br>,   <b>$name</b>,   
                                <br>   .<br><br>$@</div>" if $@;

                  $globPrint .= $tmp;
                 }

           $sth->finish();

  return $globPrint;
}

#=============================================================

sub printUnderConstruction
{
   my $self = shift;
   my $globPrint;

   my @content = $self->readFile("Skins/$skin/underConst.html");

          foreach my $line (@content)
          {
            $line =~ s/__PERSENT__/$config{'UnderConstPersent'}/eg;
            $line =~ s/__SKIN__/$skin/eg;
            $line =~ s/__XEN_VERSION__/$VERSION/eg;

            $globPrint .= $line;
          }

   return $globPrint;
}

#=============================================================

sub readAvatars
{
        my ($self, $user) = @_;
        my $globPrint;
        my @avatars;
        my %a;

        if($user)
        {
          %a = $self->db_get_hash_query("SELECT Avatar FROM ${dbPrefix}_Module_Users WHERE UserName=\"$user\"", "Avatar");
        }

        opendir (AV, "$config{'pathToWWW'}/Images/avatars") || print "Content-Type: text/html\n\n<h2>Warning:</h2> When try to open <b>$config{'pathToWWW'}/Images/avatars</b>: $!\n";
          my @content = readdir(AV);
           foreach my $line (@content)
             {
              next if $line =~ /^.$/ || $line =~ /^..$/;
              if($user)
               {
                 if($a{'Avatar'} =~ $line ) { $globPrint .= "<option selected value=\"$line\">$line</option>\n"; }
                 else { $globPrint .= "<option value=\"$line\">$line</option>\n"; }
               }
               else
               { @avatars = "<option value=\"$line\">$line</option>\n"; }

            }
        closedir(AV);

        return $globPrint if $user;
        return @avatars;
}

#=============================================================

sub statCnt #   (  Stats)
{
        my $self = shift;
        my $browser;
        my $os;

        my $hour = $self->LocalDate("\%H");
        my $minutes = $self->LocalDate("\%M");
        my $month = $self->LocalDate("\%B");
        my $year = $self->LocalDate("\%Y");

        my $in;

        my %t = $self->db_get_hash_query("SELECT Date FROM ${Libs::API::dbPrefix}_Module_Stats_Visits
                                                                                 WHERE Date=\"$month $year\"", "Date");

                                   $in = "h02" if $hour >= 0 && $hour < 2;
                                   $in = "h24" if $hour >= 2 && $hour <= 4;
                                   $in = "h46" if $hour >= 4 && $hour < 6;
                                   $in = "h68" if $hour >= 6 && $hour < 8;
                                   $in = "h810" if $hour >= 8 && $hour < 10;
                                   $in = "h1012" if $hour >= 10 && $hour < 12;
                                   $in = "h1214" if $hour >= 12 && $hour < 14;
                                   $in = "h1416" if $hour >= 14 && $hour < 16;
                                   $in = "h1618" if $hour >= 16 && $hour < 18;
                                   $in = "h1820" if $hour >= 18 && $hour < 20;
                                   $in = "h2022" if $hour >= 20 && $hour < 22;
                                   $in = "h220" if $in eq "";

                           if($t{'Date'})
                           {
                                my %t = $self->db_get_hash_query("SELECT $in FROM ${Libs::API::dbPrefix}_Module_Stats_Visits
                                                                                      WHERE Date=\"$month $year\"", $in);
                                my $cnt = $t{$in};
                                      $cnt++;

                                   $self->db_do_query("UPDATE ${Libs::API::dbPrefix}_Module_Stats_Visits SET $in = \"$cnt\"
                                                                        WHERE Date=\"$month $year\"");
                           }
                           else
                           {
                                   $self->db_do_query("INSERT INTO ${Libs::API::dbPrefix}_Module_Stats_Visits (Date, $in)
                                                                        VALUES (\"$month $year\", 1)");
                           }

                          my $userAgent = $ENV{'HTTP_USER_AGENT'};

                      #=============================================# 

                             $browser = "MSIE" if $userAgent =~ /MSIE/;
                             $browser = "Opera" if $userAgent =~ /Opera/;
                             $browser = "Netscape" if $userAgent =~ /Gecko/ || $userAgent =~ /Nav/ || $userAgent =~ /Gold/ || $userAgent =~ /X11/
                                                                || $userAgent =~ /Netscape/;
                             $browser = "Firefox" if $userAgent =~ /Firefox/;
                             $browser = "Lynx" if $userAgent =~ /Lynx/;
                             $browser = "WebTV" if $userAgent =~ /WebTV/;
                             $browser = "Konqueror" if $userAgent =~ /Konqueror/;
                             $browser = "Bot" if $userAgent =~ /Google/ || $userAgent =~ /Yandex/ || $userAgent =~ /Slurp/ || $userAgent =~ /Scooter/
                                                        || $userAgent =~ /Spider/ || $userAgent =~ /Infoseek/ || $userAgent =~ /Rambler/ || $userAgent =~ /HotBot/;
                             $browser = "Other" if $browser eq "";

                      #=============================================# 

                             $os = "Linux" if $userAgent =~ /Linux/;
                             $os = "Mac" if $userAgent =~ /Mac/ || $userAgent =~ /PPC/;
                             $os = "Windows" if $userAgent =~ /Win/;
                             $os = "FreeBSD" if $userAgent =~ /FreeBSD/;
                             $os = "SunOS" if $userAgent =~ /SunOS/;
                             $os = "IRIX" if $userAgent =~ /IRIX/;
                             $os = "AIX" if $userAgent =~ /AIX/;
                             $os = "BeOS" if $userAgent =~ /BeOS/;
                             $os = "OS2" if $userAgent =~ /OS\/2/;

                             $os = "Other" if $os eq "";

                      #=============================================#

                             %_ = ();

                        my %g = $self->db_get_hash_query("SELECT Date FROM ${Libs::API::dbPrefix}_Module_Stats_Browsers
                                                                                 WHERE Date=\"$month $year\"", "Date");

                           if($g{'Date'})
                           {
                                my %b = $self->db_get_hash_query("SELECT $browser FROM ${Libs::API::dbPrefix}_Module_Stats_Browsers
                                                                                      WHERE Date=\"$month $year\"", $browser);
                                my $cnt = $b{$browser};
                                      $cnt++;

                                   $self->db_do_query("UPDATE ${Libs::API::dbPrefix}_Module_Stats_Browsers SET $browser = \"$cnt\"
                                                                  WHERE Date=\"$month $year\"");
                           }
                           else
                           {
                                   $self->db_do_query("INSERT INTO ${Libs::API::dbPrefix}_Module_Stats_Browsers (Date, $browser)
                                                                        VALUES (\"$month $year\", 1)");
                           }

                      #=============================================#

                             %_ = ();

                        my %j = $self->db_get_hash_query("SELECT Date FROM ${Libs::API::dbPrefix}_Module_Stats_OS
                                                                                 WHERE Date=\"$month $year\"", "Date");

                           if($j{'Date'})
                           {
                                my %o = $self->db_get_hash_query("SELECT $os FROM ${Libs::API::dbPrefix}_Module_Stats_OS
                                                                                      WHERE Date=\"$month $year\"", $os);
                                my $cnt = $o{$os};
                                      $cnt++;

                                   $self->db_do_query("UPDATE ${Libs::API::dbPrefix}_Module_Stats_OS SET $os = \"$cnt\"
                                                                        WHERE Date=\"$month $year\"");
                           }
                           else
                           {
                                   $self->db_do_query("INSERT INTO ${Libs::API::dbPrefix}_Module_Stats_OS (Date, $os)
                                                                        VALUES (\"$month $year\", 1)");
                           }

                      #=============================================# Count referers (24.11.04 Nix)

                        my %r = $self->db_get_hash_query("SELECT Visitors FROM ${Libs::API::dbPrefix}_Module_Stats_Refers WHERE
                        Host=\"$ENV{'HTTP_REFERER'}\"", "Visitors");

                        if($r{'Visitors'} >= 1)
                         {
                                  my $cntVis = $_{'Visitors'}+1;
                                   $self->db_do_query("UPDATE ${Libs::API::dbPrefix}_Module_Stats_Refers SET Visitors = \"$cntVis\"
                                                                        WHERE Host=\"$ENV{'HTTP_REFERER'}\"");
                         }
                        else
                        {
                          if($ENV{'HTTP_REFERER'} !~ /$ENV{'HTTP_HOST'}/)
                           {
                                  $self->db_do_query("INSERT INTO ${Libs::API::dbPrefix}_Module_Stats_Refers (Host, Visitors)
                                                                        VALUES (\"$ENV{'HTTP_REFERER'}\", 1)") if $ENV{'HTTP_REFERER'} ne "";
                           }
                        }

                      #=============================================#

         return 0;
}

#=============================================================

sub respectHax0rs
{
  my $self = shift;

       my $ip = $ENV{'REMOTE_ADDR'};

      $self->db_do_query("INSERT INTO ${Libs::API::dbPrefix}_Module_Stats_Attacks (Date, URI, IP)
                                    VALUES (\"$localtime\", \"$ENV{'REQUEST_URI'}\", \"$ip\")");

   print "Content-Type: text/html\n\n";
   print "  <b></b>.<br>  IP   <i></i>  .\n";

   exit(9);
}

# ---------------------------------------------------------------------
# Send emails. (From YaWPS)
# ---------------------------------------------------------------------
sub sendEmail
{
        my ($self, $from, $to, $subject, $message) = @_;
        my ($x, $here, $there, $null);

        # Format input.
        $to      =~ s/[ \t]+/, /g;
        $from    =~ s/.*<([^\s]*?)>/$1/;
        $message =~ s/^\./\.\./gm;
        $message =~ s/\r\n/\n/g;
        $message =~ s/\n/\r\n/g;

        $config{'smtpServer'} =~ s/^\s+//g;
        $config{'smtpServer'} =~ s/\s+$//g;

        # Send email via SMTP.
        if ($config{mailType} == 1)
        {
                ($x, $x, $x, $x, $here)  = gethostbyname($null);
                ($x, $x, $x, $x, $there) = gethostbyname($config{smtpServer});

                my $thisserver   = pack('S n a4 x8', 2, 0,  $here);
                my $remoteserver = pack('S n a4 x8', 2, 25, $there);

                if (!(socket(S, 2, 1, 6))) { print "Socket failure. $!"; }
                if (!(bind(S, $thisserver))) { print "Bind failure. $!"; }
                if (!(connect(S, $remoteserver)))
                {
                        print "Connection to $config{smtpServer} has failed. $!";
                }

                my $oldfh = select(S);
                $| = 1;
                select($oldfh);
                $_ = <S>;
                if ($_ !~ /^220/)
                {
                        print "Sending Email: data in Connect error - 220. $!";
                }
                print S "HELO $config{smtpServer}\r\n";
                $_ = <S>;
                if ($_ !~ /^250/)
                {
                        print "Sending Email: data in Connect error - 250. $!";
                }
                print S "MAIL FROM:<$from>\n";
                $_ = <S>;
                if ($_ !~ /^250/)
                {
                        print "Sending Email: Sender address '$from' not valid. $!";
                }
                print S "RCPT TO:<$to>\n";
                $_ = <S>;
                if ($_ !~ /^250/)
                {
                        print "Sending Email: Recipient address '$to' not valid. $!";
                }
                print S "DATA\n";
                $_ = <S>;
                if ($_ !~ /^354/)
                {
                        print "Sending Email: Message send failed - 354. $!";
                }
        }

        # Send email via NET::SMTP.
        if ($config{mailType} == 2)
        {
                eval q^
                        use Net::SMTP;
                        my $smtp = Net::SMTP->new($config{'smtpServer'}, Debug => 0)
                                or print "Unable to connect to '$config{smtpServer}'. $!";

                        $smtp->mail($from);
                        $smtp->to($to);
                        $smtp->data();
                        $smtp->datasend("From: $from\n");
                        $smtp->datasend("Subject: $subject\n");
                        $smtp->datasend("\n");
                        $smtp->datasend($message);
                        $smtp->dataend();
                        $smtp->quit();
                ^;
                if ($@) { print "Net::SMTP fatal error: $@"; }
                return 1;
        }

        # Send email via sendmail.
        $ENV{PATH} = '';
        if ($config{mailType} == 0)
        {
                open S, "| $config{'Sendmail'}" or print "Mailprogram error. $!";
        }

        print S "To: $to\n";
        print S "From: $from\n";
        print S "Subject: $subject\n\n";
        print S "$message";
        print S "\n\n";

        # Send email via SMTP.
        if ($config{mailType} == 1)
        {
                $_ = <S>;
                if ($_ !~ /^250/)
                {
                        print "Sending Email: Message send failed - try again - 250. $!";
                }
                print S "QUIT\n";
        }

        close(S);
        return 1;
}


#=============================================================

sub getForms
{
  my $self = shift;
  my %FORM;

        my $buff;
        if ($ENV{REQUEST_METHOD} eq "POST") { read STDIN, $buff, $ENV{CONTENT_LENGTH} }
        else { $buff = $ENV{QUERY_STRING} }

        my @pairs = split(/&/, $buff);
        my $pair;
        foreach $pair (@pairs) {
                my ($name, $value) = split(/=/, $pair);

                $name =~ tr/+/ /;
                $name =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
                $value =~ tr/+/ /;
                $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;

                $FORM{$name} = $value;
                $FORM{$name} =~ s/\</\&lt;/g;
                $FORM{$name} =~ s/\>/\&gt;/g;
                $FORM{$name} =~ s/\"/\&quot;/g;
                $FORM{$name} =~ s/:/\&\#58;/g;
                $FORM{$name} =~ s/\'//g;
                $FORM{$name} =~ s/\`//g;
        }

  return %FORM;
}

#=============================================================

sub escape {
        my ($self, $toencode) = @_;
        $toencode=~s/([^a-zA-Z0-9_.-])/uc sprintf("%%%02x",ord($1))/eg;
        return $toencode;
}

#=============================================================

sub expires {
        my ($self, $time) = @_;
        my @MON = qw/Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec/;
        my @WDAY = qw/Sun Mon Tue Wed Thu Fri Sat/;

        $time = $self->expire_calc($time);
        return $time unless $time =~ /^\d+$/;
        my($sec,$min,$hour,$mday,$mon,$year,$wday) = gmtime($time);
        return sprintf("%s, %02d-%s-%04d %02d:%02d:%02d GMT",$WDAY[$wday],$mday,$MON[$mon],$year+1900,$hour,$min,$sec);
}

#=============================================================

sub expire_calc {
        my($self, $time) = @_;
        my(%mult) = (
                's'=>1,
                'm'=>60,
                'h'=>60*60,
                'd'=>60*60*24,
                'M'=>60*60*24*30,
                'y'=>60*60*24*365
        );
        my $offset;
        if (!$time || (lc($time) eq 'now')) { $offset = 0; }
        elsif ($time=~/^\d+/) { return $time; }
        elsif ($time=~/^([+-]?(?:\d+|\d*\.\d*))([mhdMy]?)/) { $offset = ($mult{$2} || 1)*$1; }
        else { return $time; }
        return ( time+$offset );
}

#=============================================================

sub getCookies
{
        my ($self, $nameIn) = @_;
        my @cookies = split (/\; ?/,$ENV{'HTTP_COOKIE'});
        my %cooks;

        foreach my $cooki (@cookies)
         {
                my ($name, $value) = split(/=/, $cooki);
                $name =~ tr/+/ /;
                $name =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
                $value =~ tr/+/ /;
                $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
                $cooks{$name} = $value;
         }
  return %cooks;
}

#=============================================================

sub viewDirContent
{
    my ($self, $dir, $disk) = @_;
    my $mod;
    my $content;

    my $lnt = length($dir);
    my $upper = substr $dir, 0, $lnt-2;

    if(length($upper) <= 2) { $upper = "${disk}0"; }

    ##################

   $mod = "Docs" if $disk eq "S";               #    
   $mod = "Downloads" if $disk eq "D";       #      . !

   ###################

    my @html = $self->readFile("Skins/$Libs::API::skin/fs/Explore.html");

    $content = "<td width=33% class=textbg align=center><a href=\"pages.cgi?name=$mod\&dir=$upper\">
                            <img border=0 src=\"\/Skins\/$Libs::API::skin\/images\/dir_small.png\">
                            <b>..</b></a></td>" if $dir ne "${disk}0";

         my $sth = $Libs::API::dbh->prepare("SELECT CTID, Name, IsGuestVisible FROM ${dbPrefix}_FS WHERE CTID LIKE \"\%$dir\%\"");
             eval '$sth->execute();';
             print "Content-Type: text/html\n\n<h2>Fatal error:</h2> When try to <b>execute query</b>: ".$DBI::errstr."\n" if $@;

               my $C;
               my $totalCnt;

               if($dir ne "${disk}0") { $C = 1; } else { $C = 0; }
               if($dir ne "${disk}0") { $totalCnt = 1; } else { $totalCnt = 0; }

               my $globalPrint;

               while (my $ref = $sth->fetchrow_hashref())
                 {

                 if(length($ref->{"CTID"}) == $lnt+2)
                   {
                    if(!$ref->{'IsGuestVisible'})
                     {
                     if($self->isUserValid())
                      {
                       $content .=
                    "<td width=33% class=textbg align=center><a href=\"pages.cgi?name=$mod\&dir=$ref->{'CTID'}\">
                     <img border=0 src=\"\/Skins\/$Libs::API::skin\/images\/dir_small.png\">
                     <b>$ref->{\"Name\"}</b></a></td>\n";
                     $C++;
                     $totalCnt++;
                       }
                     }
                        else
                     {
                     $content .=
                    "<td width=33% class=textbg align=center><a href=\"pages.cgi?name=$mod\&dir=$ref->{'CTID'}\">
                     <img border=0 src=\"\/Skins\/$Libs::API::skin\/images\/dir_small.png\">
                     <b>$ref->{\"Name\"}</b></a></td>\n";
                     $C++;
                     $totalCnt++;
                     }
                   }

                    if($C == 3) { if($dir ne "${disk}0") { $content .= "\n</tr><tr>\n"; $C = 1; } else { $content .= "\n</tr><tr>\n"; $C = 0; } }

                 }

                 $content .= "<td class=textbg align=center><div><b>  !</b></div</td>" if $totalCnt == 0;

                foreach my $line (@html)
                   {
                     $line =~ s/__FILES__/$content/eg;
                     $line =~ s/__MODULE__/$mod/eg;
                     $line =~ s/__TOTALDIRS__/$totalCnt/eg;
                     $line =~ s/__DIR__/$dir/g;

                     $globalPrint .= $line;
                   }

    return $globalPrint;
}

#=============================================================]

sub retchr
{
   my ($self, %hash) = @_;
   my %hash_out;

                my @keys = keys %hash;
                my @values = values %hash;

                   while (@keys)
                    {
                       my $keys = pop(@keys);
                       my $vals = pop(@values);

                       $vals =~ s/&lt;/\</g;
                       $vals =~ s/&gt;/\>/g;
                       $vals =~ s/&#58;/\:/g;
                       $vals =~ s/&quot;/\"/g;

                       $hash_out{$keys} = $vals;
                    }

   return %hash_out;
}

#=============================================================]

sub isUserValid
{
   my ($self) = shift;
   my %cook = $self->getCookies();

  if($cook{'login'} ne "" && $cook{'passwd'} ne "")
   {
    $self->db_get_hash_query("SELECT Password FROM ${Libs::API::dbPrefix}_Module_Users
                                            WHERE UserName=\"$cook{'login'}\"","Password");

    if($cook{'passwd'} eq $_{'Password'})
      {
        return 1;
      }
       else
      {
        return 0;
      }
    }
     else
      {
        return 0;
      }
}

#=============================================================]

sub isUserAdmin
{
   my ($self, $user) = @_;

    $self->db_get_hash_query("SELECT Level FROM ${Libs::API::dbPrefix}_Module_Users
                                            WHERE UserName=\"$user\"","Level");

    if($_{'Level'} eq "")
      {
        return 1;
      }
       else
      {
        return 0;
      }
}

#=============================================================
#======================== MD5 Crypting ==========================
#=============================================================

sub checkMD5
{
   my ($self, $pass, $md5) = @_;

        my $md5pass = md5_hex($pass);

        return 1 if $md5pass eq $md5;

  return 0;
}

#=============================================================

sub genMD5
{
   my ($self, $pass) = @_;

        $pass = md5_hex($pass);

  return $pass;
}

#=============================================================
#=============================================================

sub versionCheck
{
   my ($self, $line, $new) = @_;
   my $block = $line;

          my @vrs = $self->readFile("Modules/$line/version.txt");

                 chomp $vrs[0];
                 my $vrsl = substr($vrs[0], 0, 15);
                 $line .= " $vrsl";

           $self->db_do_query("INSERT INTO ${dbPrefix}_Extensions (Date, Name) VALUES (\"$localtime\", \"$line\")") if $new;
           $self->db_do_query("UPDATE ${dbPrefix}_Extensions SET Name=\"$line\" WHERE Name LIKE \"\%$block\%\"");

    return 0;
}

#=============================================================

sub LocalDate {
        my ($self, $format) = @_;
        $format = '%D' if !$format;

        my($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime;
        my $full_en_text_date = localtime;
        $mon++;
        my $yearl = $year + 1900;
        if ($year > 99) { $year -= 100; }
        my ($short_mon) = substr($Months[$mon-1],  0, 3);
        my ($short_wday) = substr($WeekDays[$wday], 0, 3);
        my $mdayl = sprintf('%02d', $mday);
        my $monl  = sprintf('%02d', $mon);
        $sec   = sprintf('%02d', $sec);
        $min   = sprintf('%02d', $min);
        $hour  = sprintf('%02d', $hour);
        $year  = sprintf('%02d', $year);

        if(length($mday) == 1) { $mday = "0$mday"; }

        $_ = $format;
        s/%%/%/g;
        s/%a/$short_wday/g;
        s/%A/$WeekDays[$wday]/g;
        s/%b|%h/$short_mon/g;
        s/%B/$Months[$mon-1]/g;
        s/%c/$full_en_text_date/g;
        s/%d/$mdayl/g;
        s/%D|%x/$monl\/$mday\/$year/g;
        s/%m/$monl/g;
        s/%H/$hour/g;
        s/%j/$yday/g;
        s/%M/$min/g;
        s/%S/$sec/g;
        s/%n/<br>/g;
        s/%X|%T/$hour:$min:$sec/g;
        s/%y/$year/g;
        s/%Y/$yearl/g;

        return $_;

        #  %a -   ().
        #  %A -  .
        #  %b -  ().
        #  %B - .
        #  %c -   (eng)
        #  %d -   ()
        #  %D  %x -    "//" (06/05/04)
        #  %m -  ().
        #  %H -  ().
        #  %M -  ().
        #  %S -  ().
        #  %n -  .
        #  %X  %T -    "::" (10:43:11)
        #  %y -  ()
        #  %Y - .
}

#=============================================================
#=============================================================

#--------------------------------------------------------------------------------------#
#------------------- Functions for work with databases ----------------#
#--------------------------------------------------------------------------------------#

sub _dbConnect
{
           my $self = shift;

           eval '
           my $dsn = "DBI:$dbDriver:database=$dbName;host=$dbHost;port=$dbPort";
                 $dbh = DBI->connect($dsn, $dbUser, $dbPass, { PrintError => 0, RaiseError => 1, AutoCommit => 1 });';
                 print "Content-Type: text/html\n\n<h2>Fatal error:</h2> When try to <b>connect</b>: ".$DBI::errstr."\n" if $@;

           return $self;
}

#=======================================================

sub _dbDisconnect
{
       my $self = shift;
       eval '$dbh->disconnect;';
       print "Content-Type: text/html\n\n<h2>Fatal error:</h2> When try to <b>disconnect</b>: ".$DBI::errstr."\n" if $@;
       return $self;
}

#=======================================================

sub db_do_query
{
       my ($self, $query) = @_;

       eval '$dbh->do($query);';
       print "Content-Type: text/html\n\n<h2>Fatal error:</h2> When try to <b>proccess query</b>: ".$DBI::errstr."\n" if $@;

       return $self;
}

#=======================================================

sub db_list_tables
{
     my $self = shift;
     my @tables;

     my $sth = $Libs::API::dbh->prepare("SHOW TABLES");

           eval '$sth->execute();';
           print "Content-Type: text/html\n\n<h2>Fatal error:</h2> When try to <b>list tables</b>: ".$DBI::errstr."\n" if $@;

               while (my $ref = $sth->fetchrow_hashref())
                 {
                        push(@tables, $ref->{"Tables_in_$dbName"});
                  }

           $sth->finish();

       return @tables;
}

#=======================================================

sub db_get_hash_query
{
       my ($self, $query, $values) = @_;

       my @value = split(/, /, $values);
       my $total = scalar(@value);

       my $sth = $dbh->prepare($query);
             eval '$sth->execute();';
             print "Content-Type: text/html\n\n<h2>Fatal error:</h2> When try to <b>execute query</b>: ".$DBI::errstr."\n" if $@;

         while (my $ref = $sth->fetchrow_hashref())
           {
             for(my $i=0;$i<=$total-1;$i++)
                {
                  $_{$value[$i]} = $ref->{$value[$i]};
                }
           }
         $sth->finish();

       return %_;
}

#==========================================================

sub db_rows_count
{
    my ($self, $query) = @_;

        my $sth = $dbh->prepare($query);
              eval '$sth->execute();';
              print "Content-Type: text/html\n\n<h2>Fatal error:</h2> When try to <b>execute query</b>: ".$DBI::errstr."\n" if $@;

                  my $rv = $sth->rows();

             $sth->finish();

    return $rv;
}

#==========================================================

#--------------------------------------------------------------------------------------#
#---------------------------- End functions ------------------------------------#
#--------------------------------------------------------------------------------------#

1;

__END__

=head1 USAGE

        my $api = new Libs::API ( initMods => 0, readConf => 1 )
             $api->printHeader("PageTitle");
              ....

=head2 initAllModules

       $api->initAllModules();

       Initializing and loading all existing founded modules.
       If we have a modules not registred on DB, then write
       it on DB with him number and current date.

=head2 singleInit

       $api->singleInit("ModuleDirName", (1|0));

       Initializing and loading all existing founded modules
       on directory. If second parametr is true, then loading
       all founded modules at admin directory.

=head2 readConfig

       $api->readConfig(TableName);

       Extracting and pushing data to hash from $dbPrefix_TableName.

=head2 printHeader

       $api->printHeader("Page Title");

       Printering all headers, including Content-Type and
       meta-tags with your info.

=head2 printFooter

       $api->printFooter();

       Printering footer part of page.

=head2 printBody

       $api->printBody($leftBlock, $rightBlock, $centerBlock);

       Printering all of 3 parts of page - left, right and center.

=head2 printBlock

       $api->printBlock("BlockTitle", "BlockContent", "BlockType");

       Printering simple one block.
       Example: Block Type = 1 => /Skins/Default/block-1.tmp

=head1 ENGLISH

        Sorry for my awfull english =)

=cut
